// wsnote.h
// e.moon 18dec98

#ifndef __EMOON_WSNOTE_H__
#define __EMOON_WSNOTE_H__

class App;
class Window;
class View;

class View : public BView {
	typedef BView _inherited;
	
public:
	static const rgb_color		s_bgColor;
	static const rgb_color		s_bgColorHilight;
	static const rgb_color		s_textColor;
	static const rgb_color		s_borderColor;
	static const rgb_color		s_hilightColor;
	
	static const uint32				s_nBorderSize;
	static const uint32				s_nCornerSize;
	static const uint32				s_nScreenSnapThreshold;
	static const uint32				s_nMinWidth;
	static const uint32				s_nMinHeight;

	enum dragtype {
		DRAG_NONE,
		DRAG_MOVE,
		DRAG_UL,
		DRAG_TOP,
		DRAG_UR,
		DRAG_RIGHT,
		DRAG_LR,
		DRAG_BOTTOM,
		DRAG_LL,
		DRAG_LEFT
	};

	char 					buffer[16];
	BFont					m_font;
	font_height		m_fontHeight;
	
	// current drag op
	dragtype			m_drag;
	dragtype			m_dragHilight;
	BPoint				m_dragInitPoint;
	BRect					m_dragInitFrame;
	
	// frame resize mech.
	BRect					m_prevBounds;
	bool						m_bSnapEdge;

	// context menu
	BPopUpMenu*		m_pMenu;
	
public:
	View(BRect frame, bool bSnap);
	
	void AllAttached();
	
	void Draw(BRect updateRect);
	void drawDragHilight(BRect b);
	
	void FrameResized(float fWidth, float fHeight);
	
	void MouseDown(BPoint p);
	void MouseUp(BPoint p);
	void MouseMoved(BPoint p, uint32 nTransit,
		const BMessage* pDragMsg);
		
	bool edgeSnap() const { return m_bSnapEdge; }
	void setEdgeSnap(bool bSnap) {
		m_bSnapEdge = bSnap;
	}

protected:
	// update drag operation
	void dragUpdate(BPoint p);
	// given hilight style, display a hint
	void updateDragHilight(dragtype t);
	// given point, figure drag type
	dragtype dragType(BPoint p);
	
	// given current window frame rect,
	// 'snap' to screen edge if within threshold
	void snapToScreenEdge(BRect frame, BPoint& oDelta);
	
	// given window frame & delta, constrain resize
	// operation to minimum window size
	void constrainResize(BRect frame, BPoint& ioDelta);
	
	// show the 'About' box
	void doAbout();
};

// -------------------------------------------------- //
// bare-bones window class
// -------------------------------------------------- //


class Window : public BWindow {
	typedef BWindow _inherited;

public:
	View*		m_pView;
	
public:
	Window(BRect frame, bool bSnap) :
		BWindow(frame, "wsnote", B_NO_BORDER_WINDOW_LOOK,
			B_FLOATING_ALL_WINDOW_FEEL,
			B_WILL_ACCEPT_FIRST_CLICK,
			B_ALL_WORKSPACES) {
		m_pView = new View(Bounds(), bSnap); 
		AddChild(m_pView);
	}
	
	bool QuitRequested();
	
	void WorkspaceActivated(int32 nWorkspace, bool bActive);
	void MessageReceived(BMessage* pMsg);

protected:
	void doAbout();
	void toggleSnap(BMessage* pMsg);
};

// -------------------------------------------------- //
// app class
// -------------------------------------------------- //

class App : public BApplication {
public:
	App() :
		BApplication("application/x-vnd.emoon-wsnote-1.0") {
		BRect r; bool bSnap;
		fetchSettings(r, bSnap);
		m_pWnd = new Window(r, bSnap);
		m_pWnd->Show();
	}

	void fetchSettings(BRect& r, bool& bSnap);
	void storeSettings(BRect r, bool bSnap);

public:
	static const int32 s_nDefWndWidth;
	static const int32 s_nDefWndHeight;
	
	BWindow*		m_pWnd;
};



#endif /* __EMOON_WSNOTE_H__ */